#!/usr/bin/env python
#
# Problem EPage 173
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
from numpy import cos, sin, linspace
from newtonRaphson import *

def f(x): return sin(x) + 3 * cos(x) - 2 

def secant(f,a,b,tol=1.e-9):
    """
    Implements the secant method.  Code modified from newtonRaphson.py
    """
    fa = f(a)
    if fa == 0.0: return a
    fb = f(b)
    if fb == 0.0: return b
    if fa*fb > 0.0: print 'Root is not bracketed'
    x = 0.5*(a + b)                    
    for i in range(1000):
        fx = f(x)
        if abs(fx) < tol: return x
        # Try a secant step
        dfx_secant = (fb-fa)/(b-a)
        # Tighten the brackets on the root 
        if fa*fx < 0.0:
            b = x  
        else:                  
            a = x
        # If division by zero, push x out of bounds
        try: dx = -fx/dfx_secant
        except ZeroDivisionError: dx = b - a
        x = x + dx
        # If the result is outside the brackets, use bisection  
        if (b - x)*(x - a) < 0.0:  
            dx = 0.5*(b - a)                      
            x = a + dx
        # Check for convergence     
        if abs(dx) < tol*max(abs(b),1.0): return x
    print 'Too many iterations in secant'


if __name__ == "__main__":
    if False:
        xs = linspace( -2, +2, 100 ) 
        fs = [ f(_) for _ in xs ]
    
        import pylab; 
        pylab.plot( xs, fs ); 
        pylab.xlabel('x'); pylab.ylabel('f(x)');
        pylab.hold('on')
        pylab.axhline(y=0,color='k')
        pylab.grid('on'); 
        pylab.show()
        
    print "Leftmost root= ", secant( f, -0.75, -0.25 )
    print "Rightmost root= ", secant( f, +1.0, +1.5 )


