#!/usr/bin/env python
#
from numpy import sin, cos

# Problem EPage 205 (estimate derivatives of sin(x) at x=0.8)

x_0 = 0.8
d_exact = cos(x_0) 

hs = [ 1, 0.5, 0.25, 0.1, 0.05, 0.01, 0.001, 1.e-4, 5.e-5, 1.e-5, 1.e-6 ]

def d_sin_x_forward( h, x_0=0.8, n_significant_digits=5 ):
    """
    computes an estimate of d ( sin(x) ) / d x using

    ( sin(x+h) - sin(x) ) / h 

    but with only n_significant_digits at each stage of the calculation.
    """
    
    pt_1 = round( sin( x_0 + h ), n_significant_digits )
    pt_2 = round( sin( x_0 ), n_significant_digits )
    fp = round( ( pt_1 - pt_2 ) / h, n_significant_digits )
    return fp 

def d_sin_x_central( h, x_0=0.8, n_significant_digits=5 ):
    """
    computes an estimate of d ( sin(x) ) / d x using

    ( sin(x+h) - sin(x-h) ) / 2 h 

    but with only n_significant_digits at each stage of the calculation.
    """
    
    pt_1 = round( sin( x_0 + h ), n_significant_digits )
    pt_2 = round( sin( x_0 - h ), n_significant_digits )
    fp = round( ( pt_1 - pt_2 ) / (2*h), n_significant_digits )
    return fp 
    

print "First forward estimates:"
print "%10s %10s" % ("h","error")
for h in hs : 
    d_approx = d_sin_x_forward(h)
    error = d_exact - d_approx
    print "%10.6f %10.6f" % (h,error)


print "First central estimates:"
print "%10s %10s" % ("h","error")
for h in hs : 
    d_approx = d_sin_x_central(h)
    error = d_exact - d_approx
    print "%10.6f %10.6f" % (h,error)
