#!/usr/bin/env python
#
from numpy import asarray, nan, empty, sin, cos, tan, pi

# Problem EPage 205-206

times = [ 9, 10, 11 ]
alphas = asarray( [ 54.80, 54.06, 53.34 ] ) # in degrees
betas = asarray( [ 65.59, 64.59, 63.62 ] ) 

alphas *= ( pi / 180. ) # in radians 
betas *= ( pi / 180. ) 

delta_t = 1

def sec(x) : return 1./cos(x)

def fp_approx(x,delta_t=1):
    """
    Returns an estimate of x'(t) using forward/central/backwards differences of O(h^2) 
    """
    n = len(x)
    x_p = nan*empty(n)

    # Use forward differences for the first location (Table 5.3a):
    # 
    for ii in [0]: 
        x_p[ii] = ( - 3 * x[ii] + 4 * x[ii+1] - x[ii+2] ) / ( 2 * delta_t ) 

    # Use central differences for the middle locations (Table 5.1):
    # 
    for ii in range(1,n-1): 
        x_p[ii] = ( - x[ii-1] + x[ii+1] ) / ( 2 * delta_t ) 

    # Use backwards differences for the last location (Table 5.3b):
    # 
    for ii in [n-1]: 
        x_p[ii] = ( x[ii-2] - 4 * x[ii-1] + 3 * x[ii] ) / ( 2 * delta_t ) 

    return x_p

a = 500 # in meters 
alpha_dot = fp_approx(alphas)
beta_dot = fp_approx(betas)

#
# Use the formalas for dx/dt and dy/dt in terms of dalpha/dt and dbeta/dt to compute dx/dt and dy/dt: 
# 
denom = tan(betas) - tan(alphas)

x_dot = ( -tan(alphas) * ( sec(alphas)**2 ) * beta_dot + tan(betas) * ( sec(alphas)**2 ) * alpha_dot ) / ( denom**2 )
x_dot *= a 
print "x_dot= ", x_dot

y_dot = ( ( sec(alphas)**2 ) * ( tan(betas)**2 ) * alpha_dot - ( sec(betas)**2 ) * ( tan(alphas)**2 ) * beta_dot ) / ( denom**2 )
y_dot *= a 
print "y_dot= ", y_dot

 


 




