#!/usr/bin/env python
#
# Problem EPage 239
# Table of Gauss-Legendre points on EPage 230
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
from numpy import sqrt, asarray
from gaussQuad import *
from polyFit import * 
from evalPoly import evalPoly 

hs = asarray( [ 112, 80, 52, 35, 15, 0 ] )
ps = asarray( [ 620, 612, 575, 530, 425, 310 ] )

# Fit a polynomial through these data points:
# 
coefs = polyFit( hs, ps, 3 )

def p_fn_of_h(h):
    """
    evaluates the fitted polynomial 
    """
    result, _, _ = evalPoly(coefs,h)
    return result 

def f_numer(h): return h * p_fn_of_h(h)

def f_denom(h): return p_fn_of_h(h)

def nintegrate( f, a, b, tol=1.e-5, max_num_nodes=500 ):
    """
    """
    I_old = gaussQuad( f, a, b, 2 ) 
    for m in range(2+1,max_num_nodes+1):
        I_new = gaussQuad( f, a, b, m ) 
        if abs( I_new - I_old ) < tol * max(abs(I_new),1.0) : 
            return I_new, m  
        I_old = I_new
    print "Gauss-Legendre did not converge to desired tolerance using fewer than %5d nodes" % max_num_nodes

if __name__ == "__main__":
    I_numer = nintegrate( f_numer, 0, 112 )
    I_denom = nintegrate( f_denom, 0, 112 )
    print "Pressure Center height= %10.6f" % (I_numer[0] / I_denom[0])

