#!/usr/bin/env python
#
# Problem Page 308
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
from numpy import zeros,array,pi
import pylab

from run_kut5 import integrate
from ridder import *
from printSoln import *

def initCond(u):  # Init. values of [y,y']; use 'u' if unknown
    return array([1.0, u])  

def r(u):         # Boundary condition residual--see Eq. (8.3)
    X,Y = integrate(F,xStart,initCond(u),xStop,h)
    y = Y[len(Y) - 1]
    r = y[1] # y'(2) = 0
    return r

def F(x,y):       # First-order differential equations
    F = zeros(len(y))
    F[0] = y[1]
    F[1] = -y[1]/x - y[0] 
    return F

xStart = 0.01       # Start of integration
xStop = 2.0         # End of integration
u1 = -100.0         # 1st trial value of unknown init. cond.
u2 = +0.0           # 2nd trial value of unknown init. cond.
h = 0.1             # Step size
freq = 2            # Printout frequency
u = ridder(r,u1,u2) # Compute the correct initial condition
X,Y = integrate(F,xStart,initCond(u),xStop,h)
printSoln(X,Y,freq)

if True:
    pylab.plot( X, Y[:,0], color='black', label='approx. sol. y(x)' )
    pylab.hold('on')
    pylab.plot( X, Y[:,1], color='blue', label="approx. sol. y'(x)" )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.legend(loc='best')
    pylab.show() 
