#!/usr/bin/env python
#
# Problem Page 308
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
from numpy import zeros,array,pi
import pylab

from run_kut5 import integrate
from ridder import *
from printSoln import *

def initCond(u):  # Init. values of [y,y']; use 'u' if unknown
    return array([0.0, u])  

def r(u):         # Boundary condition residual--see Eq. (8.3)
    X,Y = integrate(F,xStart,initCond(u),xStop,h)
    y = Y[len(Y) - 1]
    r = y[0] - 1.0 
    return r

def F(x,y):       # First-order differential equations
    F = zeros(len(y))
    F[0] = y[1]
    F[1] = -(1-0.2*x)*y[0]**2
    return F

xStart = 0.0        # Start of integration
xStop = pi/2.       # End of integration
u1 = 0.5
u2 = 1.0 
h = 0.1             # Step size
freq = 1            # Printout frequency
u = ridder(r,u1,u2) # Compute the correct initial condition
X,Y = integrate(F,xStart,initCond(u),xStop,h)
printSoln(X,Y,freq)

if True: 
    pylab.plot( X, Y[:,0], color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.legend(loc='best')
    pylab.show() 
