#!/usr/bin/python
#
from numpy import zeros,ones,array,arange
from LUdecomp3 import *
from math import pi

#
# y''(x) = (2+x) y
# y(0) = 0 
# y'(1) = 5 -> y_{m+1} - y_{m-1} = 5 (2h)
# 

def equations(x,h,m): # Set up finite difference eqs.
    h2 = h*h

    # Form the parts of A: 
    c = ones(m)
    d = (-2 - 2*(2+x)*h2) * ones(m+1)
    e = ones(m)
    d[0] = 1.0 # first equation is: y_0 = 0 
    e[0] = 0.0 
    c[m-1] = 2.0 # from the boundary conditions y'(1) = 5 

    # Form the parts of b: 
    b = zeros(m+1)
    b[m] = -10*h

    return c,d,e,b

xStart = 0.0        # x at left end
xStop = 1.0         # x at right end
m = 1000            # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
c,d,e,b = equations(x,h,m)
c,d,e = LUdecomp3(c,d,e)
y = LUsolve3(c,d,e,b)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])
print "m= %5d; y'(1)= %6.3f" % ( m, (y[m]-y[m-1])/(x[m]-x[m-1]) )

import pylab 

if True: 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
