#!/usr/bin/python
#
import numpy as np; from numpy import zeros,ones,array,arange
from LUdecomp5 import *

gamma = 10.**5

def equations(x,h,m,verbose=False): # Set up finite difference eqs.
    """
    y_{i-2} - 4 y_{i-1} + ( 6 + \gamma h^4 ) y_i - 4 y_{i+1} + y_{i+2} = h^4 

    Note this system is not symmetric since the boundary conditions make A[0,1] = 0 != A[1,0]

    Thus we can't use LUdecomp5 and will instead invert a full matrix.
    """
    h2 = h*h

    A = zeros( (m+1,m+1) ) 

    # fill in the diagonal elements: 
    i = 0
    A.ravel()[i:max(0,A.shape[1]-i)*A.shape[1]:A.shape[1]+1] = ( 6 + gamma * h2**2 ) * ones(m+1)

    # fill in the ith super/sub diagonal:
    e = -4 * ones(m)
    i = +1 
    A.ravel()[i:max(0,A.shape[1]-i)*A.shape[1]:A.shape[1]+1] = e.copy() 
    A.ravel()[A.shape[1]*i:A.shape[1]*(i+A.shape[1]):A.shape[1]+1] = e.copy() 

    # fill in the ith super/sub diagonal:
    f = ones(m-1)
    i = +2 
    A.ravel()[i:max(0,A.shape[1]-i)*A.shape[1]:A.shape[1]+1] = f.copy() 
    A.ravel()[A.shape[1]*i:A.shape[1]*(i+A.shape[1]):A.shape[1]+1] = f.copy() 

    # first equation is: y_0 = 0:
    A[0,0] = 1.0
    A[0,1] = 0.0 
    A[0,2] = 0.0 

    # the second equation is: -2 y_0 + ( 5 + gamma h^4 ) y_1 - 4 y_2 + y_3 = h^4 
    A[1,0] = -2
    A[1,1] = 5 + gamma * h2**2

    # the second to last equation is: y_{m-3} - 4 y_{m-2} + ( 5 + gamma h^4 ) y_{m-1} - 2 y_m = h^4
    A[m-1,m-1] = 5 + gamma * h2**2
    A[m-1,m] = -2

    # last equation is: y_m = 0 
    A[m,m-2] = 0.0 
    A[m,m-1] = 0.0
    A[m,m] = 1.0

    if verbose: 
        print "A= ", A 

    # Form the parts of b: 
    b = h2**2 * ones(m+1) 
    b[0] = 0.0
    b[m] = 0.0 
    return A,b

xStart = 0.0        # x at left end
xStop = 1.0         # x at right end
m = 1000            # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
A,b = equations(x,h,m)
y = np.linalg.solve(A,b) 
print "m= %5d" % m 
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])
print "y'(1)= %10.6f" % ( (y[-1]-y[-2])/(x[-1]-x[-2]) )

if True: 
    import pylab 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
