#!/usr/bin/python
#
from numpy import zeros,ones,array,arange
from LUdecomp3 import *
from math import pi

#
# y''(x) = x y
# y(1) = 1.5
# y(2) = 3.0
# 

def equations(x,h,m): # Set up finite difference eqs.
    """
    y_{i-1} + ( -2 - x_i h^2 ) y_i + y_{i+1} = 0 
    """
    h2 = h*h

    # Form the parts of A = [c\d\e]:
    c = ones(m)
    d = (-2 - x*h2) * ones(m+1)
    e = ones(m)
    d[0] = 1.0 # first equation is: y_0 = 1.5
    e[0] = 0.0 
    d[m] = 1.0 # last equation is: y_m = 3.0
    c[m-1] = 0.0 

    # Form the parts of b: 
    b = zeros(m+1) 
    b[0] = 1.5
    b[m] = 3.0 
    return c,d,e,b

xStart = 1.0        # x at left end
xStop = 2.0         # x at right end
m = 10              # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
c,d,e,b = equations(x,h,m)
c,d,e = LUdecomp3(c,d,e)
y = LUsolve3(c,d,e,b)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])

import pylab 

if True: 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
