#!/usr/bin/python
#
from numpy import zeros,ones,array,arange,log,sin
from LUdecomp3 import *

#
# y''(x) = - x y' - y 
# y(1) = 0
# y(2) = 0.638961 
#
# Exact solution
#
# y = sin(ln(x))
# 

def equations(x,h,m): # Set up finite difference eqs.
    """
    ( x_i^2 - \frac{ x_i h }{ 2 } ) y_{i-1} + ( h^2 - 2 x_i^2 ) y_i + ( x_i^2 + \frac{ x_i h }{ 2 } ) y_{i+1} = 0 
    """
    h2 = h*h

    # Form the parts of A = [c\d\e]:
    c = ( x[1:]**2 - x[1:] * h / 2 ) * ones(m)
    d = ( h2 - 2 * x**2 ) * ones(m+1)
    e = ( x[0:-1]**2 + x[0:-1] * h / 2 ) * ones(m)
    d[0] = 1.0 # first equation is: y_0 = 0
    e[0] = 0.0 
    d[m] = 1.0 # last equation is: y_m = 0.638961 
    c[m-1] = 0.0 

    # Form the parts of b: 
    b = zeros(m+1) 
    b[0] = 0.0
    b[m] = 0.638961 
    return c,d,e,b

xStart = 1.0        # x at left end
xStop = 2.0         # x at right end
m = 100              # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
c,d,e,b = equations(x,h,m)
c,d,e = LUdecomp3(c,d,e)
y = LUsolve3(c,d,e,b)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])

import pylab 

if True: 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.hold('on')
    pylab.plot( x, sin(log(x)), label='true sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
