## module powerMethod
''' lam,x = powerMethod(a,tol=1.0e-6).
    The power method for solving the eigenvalue problem
    [a]{x} = lam{x}. Returns the largest eigenvector and the
    corresponding eigenvector {x}.
'''
from numpy import zeros,dot,identity,sqrt,random

def powerMethod(a,tol=1.0e-9):
    n = len(a)
    x = zeros(n)
    for i in range(n):          # Seed [x] with random numbers
        x[i] = random.normal()
    xMag = sqrt(dot(x,x))       # Normalize [x]
    x = x/xMag
    for i in range(50):         # Begin iterations      
        xOld = x.copy()         # Save current [x]
        x = dot(a,xOld) 
        xMag = sqrt(dot(x,x))   # Normalize [x]
        x = x/xMag
        if dot(xOld,x) < 0.0:   # Detect change in sign of [x]
            sign = -1.0
        else:
            sign = 1.0
        error = sqrt(dot(xOld - x,xOld - x))/xMag
        if error < tol: return sign*xMag, x
    print 'power method did not converge'


    
    
